@@ -57,8 +57,8 @@
  * #GdkDeviceManager implementation will be used as the input source.
  * Otherwise either the core or XInput 1 implementations will be used.
  *
- * For simple applications that don’t have any special interest in
- * input devices, the so-called “client pointer”
+ * For simple applications that don't have any special interest in
+ * input devices, the so-called <firstterm>client pointer</firstterm>
  * provides a reasonable approximation to a simple setup with a single
  * pointer and keyboard. The device that has been set as the client
  * pointer can be accessed via gdk_device_manager_get_client_pointer().
@@ -75,11 +75,10 @@
  *
  * There may be several virtual devices, and several physical devices could
  * be controlling each of these virtual devices. Physical devices may also
- * be “floating”, which means they are not attached to any virtual device.
+ * be "floating", which means they are not attached to any virtual device.
  *
- * # Master and slave devices
- *
- * |[
+ * <example><title>Master and slave devices</title>
+ * <screen>
  * carlos@sacarino:~$ xinput list
  * ⎡ Virtual core pointer                          id=2    [master pointer  (3)]
  * ⎜   ↳ Virtual core XTEST pointer                id=4    [slave  pointer  (2)]
@@ -96,7 +95,8 @@
  *     ↳ Integrated Camera                         id=9    [slave  keyboard (3)]
  *     ↳ AT Translated Set 2 keyboard              id=12   [slave  keyboard (3)]
  *     ↳ ThinkPad Extra Buttons                    id=15   [slave  keyboard (3)]
- * ]|
+ * </screen>
+ * </example>
  *
  * By default, GDK will automatically listen for events coming from all
  * master devices, setting the #GdkDevice for all events coming from input
@@ -110,17 +110,6 @@
  * it is possible to get the source (slave) device that the event originated
  * from via gdk_event_get_source_device().
  *
- * On a standard session, all physical devices are connected by default to
- * the "Virtual Core Pointer/Keyboard" master devices, hence routing all events
- * through these. This behavior is only modified by device grabs, where the
- * slave device is temporarily detached for as long as the grab is held, and
- * more permanently by user modifications to the device hierarchy.
- *
- * On certain application specific setups, it may make sense
- * to detach a physical device from its master pointer, and mapping it to
- * an specific window. This can be achieved by the combination of
- * gdk_device_grab() and gdk_device_set_mode().
- *
  * In order to listen for events coming from devices
  * other than a virtual device, gdk_window_set_device_events() must be
  * called. Generally, this function can be used to modify the event mask
@@ -135,13 +124,9 @@
  * changes, the #GdkDevice:n-axes property will be notified, and
  * gdk_device_list_axes() will return the new device axes.
  *
- * Devices may also have associated “keys” or
+ * Devices may also have associated <firstterm>keys</firstterm> or
  * macro buttons. Such keys can be globally set to map into normal X
  * keyboard events. The mapping is set using gdk_device_set_key().
- *
- * In GTK+ 3.20, a new #GdkSeat object has been introduced that
- * supersedes #GdkDeviceManager and should be preferred in newly
- * written code.
  */
 
 static void gdk_device_manager_set_property (GObject      *object,
@@ -203,7 +188,7 @@ gdk_device_manager_class_init (GdkDeviceManagerClass *klass)
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (GdkDeviceManagerClass, device_added),
                   NULL, NULL,
-                  NULL,
+                  g_cclosure_marshal_VOID__OBJECT,
                   G_TYPE_NONE, 1,
                   GDK_TYPE_DEVICE);
 
@@ -222,7 +207,7 @@ gdk_device_manager_class_init (GdkDeviceManagerClass *klass)
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (GdkDeviceManagerClass, device_removed),
                   NULL, NULL,
-                  NULL,
+                  g_cclosure_marshal_VOID__OBJECT,
                   G_TYPE_NONE, 1,
                   GDK_TYPE_DEVICE);
 
@@ -248,7 +233,7 @@ gdk_device_manager_class_init (GdkDeviceManagerClass *klass)
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (GdkDeviceManagerClass, device_changed),
                   NULL, NULL,
-                  NULL,
+                  g_cclosure_marshal_VOID__OBJECT,
                   G_TYPE_NONE, 1,
                   GDK_TYPE_DEVICE);
 }
@@ -299,9 +284,9 @@ gdk_device_manager_get_property (GObject      *object,
  *
  * Gets the #GdkDisplay associated to @device_manager.
  *
- * Returns: (nullable) (transfer none): the #GdkDisplay to which
- *          @device_manager is associated to, or %NULL. This memory is
- *          owned by GDK and must not be freed or unreferenced.
+ * Returns: (transfer none): the #GdkDisplay to which @device_manager is
+ *          associated to, or #NULL. This memory is owned by GDK and
+ *          must not be freed or unreferenced.
  *
  * Since: 3.0
  **/
@@ -322,14 +307,11 @@ gdk_device_manager_get_display (GdkDeviceManager *device_manager)
  * @device_manager.
  *
  * Returns: (transfer container) (element-type Gdk.Device): a list of 
- *          #GdkDevices. The returned list must be
+ *          #GdkDevice<!-- -->s. The returned list must be
  *          freed with g_list_free (). The list elements are owned by
  *          GTK+ and must not be freed or unreffed.
  *
  * Since: 3.0
- *
- * Deprecated: 3.20, use gdk_seat_get_pointer(), gdk_seat_get_keyboard()
- *             and gdk_seat_get_slaves() instead.
  **/
 GList *
 gdk_device_manager_list_devices (GdkDeviceManager *device_manager,
@@ -348,15 +330,13 @@ gdk_device_manager_list_devices (GdkDeviceManager *device_manager,
  * for this application. In X11, window managers may change this depending on the interaction
  * pattern under the presence of several pointers.
  *
- * You should use this function seldomly, only in code that isn’t triggered by a #GdkEvent
- * and there aren’t other means to get a meaningful #GdkDevice to operate on.
+ * You should use this function seldomly, only in code that isn't triggered by a #GdkEvent
+ * and there aren't other means to get a meaningful #GdkDevice to operate on.
  *
  * Returns: (transfer none): The client pointer. This memory is
  *          owned by GDK and must not be freed or unreferenced.
  *
  * Since: 3.0
- *
- * Deprecated: 3.20: Use gdk_seat_get_pointer() instead.
  **/
 GdkDevice *
 gdk_device_manager_get_client_pointer (GdkDeviceManager *device_manager)
